@@ -14,12 +14,18 @@
 #' @param gp_coords A \code{matrix} with coordinates (features) for Gaussian process
 #' @param gp_rand_coef_data A \code{vector} or \code{matrix} with covariate data for Gaussian process random coefficients
 #' @param cov_function A \code{string} specifying the covariance function for the Gaussian process. 
-#' The following covariance functions are available: "exponential", "gaussian", "matern", and "powered_exponential". 
-#' We follow the notation and parametrization of Diggle and Ribeiro (2007) except for the Matern covariance 
-#' where we follow Rassmusen and Williams (2006)
-#' @param cov_fct_shape A \code{numeric} specifying the shape parameter of a covariance function 
-#' (=smoothness parameter for Matern covariance, irrelevant for some covariance functions 
-#' such as the exponential or Gaussian)
+#' The following covariance functions are available:
+#' "exponential", "gaussian", "matern", "powered_exponential", "wendland", and "exponential_tapered".
+#' For "exponential", "gaussian", and "powered_exponential", we follow the notation and parametrization of Diggle and Ribeiro (2007).
+#' For "matern", we follow the notation of Rassmusen and Williams (2006).
+#' For "wendland", we follow the notation of Bevilacqua et al. (2019).
+#' A covariance function with the suffix "_tapered" refers to a covariance function that is multiplied by 
+#' a compactly supported Wendland covariance function (= tapering)
+#' @param cov_fct_shape A \code{numeric} specifying the shape parameter of the covariance function 
+#' (=smoothness parameter for Matern and Wendland covariance). For the Wendland covariance function, 
+#' we follow the notation of Bevilacqua et al. (2019)). 
+#' This parameter is irrelevant for some covariance functions such as the exponential or Gaussian.
+#' @param cov_fct_taper_range A \code{numeric} specifying the range parameter of the Wendland covariance function / taper. We follow the notation of Bevilacqua et al. (2019)
 #' @param vecchia_approx A \code{boolean}. If true, the Vecchia approximation is used 
 #' @param num_neighbors An \code{integer} specifying the number of neighbors for the Vecchia approximation
 #' @param vecchia_ordering A \code{string} specifying the ordering used in the Vecchia approximation. 
@@ -42,12 +48,11 @@
 #' @param params A \code{list} with parameters for the model fitting / optimization
 #'             \itemize{
 #'                \item{optimizer_cov}{ Optimizer used for estimating covariance parameters. 
-#'                Options: "gradient_descent" or "fisher_scoring". Default="fisher_scoring" for Gaussian data
-#'                and "gradient_descent" for other likelihoods.}
+#'                Options: "gradient_descent", "fisher_scoring", and "nelder_mead". Default = "gradient_descent"}
 #'                \item{optimizer_coef}{ Optimizer used for estimating linear regression coefficients, if there are any 
 #'                (for the GPBoost algorithm there are usually none). 
-#'                Options: "gradient_descent" or "wls". Gradient descent steps are done simultaneously 
-#'                with gradient descent steps for the covariance paramters. 
+#'                Options: "gradient_descent", "wls", and "nelder_mead". Gradient descent steps are done simultaneously 
+#'                with gradient descent steps for the covariance parameters. 
 #'                "wls" refers to doing coordinate descent for the regression coefficients using weighted least squares.
 #'                Default="wls" for Gaussian data and "gradient_descent" for other likelihoods.}
 #'                \item{maxit}{ Maximal number of iterations for optimization algorithm. Default=1000.}
@@ -69,8 +74,7 @@
 #'                Default=0.5.}
 #'                \item{momentum_offset}{ Number of iterations for which no mometum is applied in the beginning.
 #'                Default=2.}
-#'                \item{trace}{ If TRUE, the value of the gradient is printed for some iterations.
-#'                Useful for finding good learning rates. Default=FALSE.}
+#'                \item{trace}{ If TRUE, information on the progress of the parameter optimization is printed. Default=FALSE.}
 #'                \item{convergence_criterion}{ The convergence criterion used for terminating the optimization algorithm.
 #'                Options: "relative_change_in_log_likelihood" (default) or "relative_change_in_parameters".}
 #'                \item{std_dev}{ If TRUE, (asymptotic) standard deviations are calculated for the covariance parameters}
@@ -95,7 +99,10 @@ gpb.GPModel <- R6::R6Class(
       if (!gpb.is.null.handle(private$handle)) {
         
         # Freeing up handle
-        gpb.call("GPB_REModelFree_R", ret = NULL, private$handle)
+        .Call(
+          GPB_REModelFree_R
+          , private$handle
+        )
         private$handle <- NULL
         
       }
@@ -111,6 +118,7 @@ gpb.GPModel <- R6::R6Class(
                           gp_rand_coef_data = NULL,
                           cov_function = "exponential",
                           cov_fct_shape = 0,
+                          cov_fct_taper_range = 1,
                           vecchia_approx = FALSE,
                           num_neighbors = 30L,
                           vecchia_ordering = "none",
@@ -120,7 +128,7 @@ gpb.GPModel <- R6::R6Class(
                           free_raw_data = FALSE,
                           modelfile = NULL,
                           model_list = NULL) {
-
+      
       if (!is.null(modelfile) | !is.null(model_list)){
         # Load model from file or list
         
@@ -147,8 +155,8 @@ gpb.GPModel <- R6::R6Class(
           if (!is.null(model_list[[feature]])) {
             if (is.list(model_list[[feature]])) {
               model_list[[feature]] <- matrix(unlist(model_list[[feature]]),
-                                             nrow = length(model_list[[feature]]),
-                                             byrow = TRUE)
+                                              nrow = length(model_list[[feature]]),
+                                              byrow = TRUE)
               if (dim(model_list[[feature]])[2]==1) {
                 model_list[[feature]] <- as.vector(model_list[[feature]])
               }
@@ -164,6 +172,7 @@ gpb.GPModel <- R6::R6Class(
         gp_rand_coef_data = model_list[["gp_rand_coef_data"]]
         cov_function = model_list[["cov_function"]]
         cov_fct_shape = model_list[["cov_fct_shape"]]
+        cov_fct_taper_range = model_list[["cov_fct_taper_range"]]
         vecchia_approx = model_list[["vecchia_approx"]]
         num_neighbors = model_list[["num_neighbors"]]
         vecchia_ordering = model_list[["vecchia_ordering"]]
@@ -225,13 +234,14 @@ gpb.GPModel <- R6::R6Class(
         group_data_unique <- unique(group_data)
         group_data_unique_c_str <- lapply(group_data_unique,gpb.c_str)
         group_data_c_str <- unlist(group_data_unique_c_str[match(group_data,group_data_unique)])
-        # group_data_c_str <- unlist(lapply(group_data,gpb.c_str))# Version 2: slower than above
+        # Version 2: slower than above
+        # group_data_c_str <- unlist(lapply(group_data,gpb.c_str))
         # group_data_c_str <- c()# Version 3: much slower
         # for (i in 1:length(group_data)) {
         #   group_data_c_str <- c(group_data_c_str,gpb.c_str(group_data[i]))
         # }
         
-        # Set data for random coef for grouped REs
+        # Set data for random coefficients for grouped REs
         if (!is.null(group_rand_coef_data)) {
           
           if (is.numeric(group_rand_coef_data)) {
@@ -322,45 +332,22 @@ gpb.GPModel <- R6::R6Class(
           
         }
         
-        if (!(vecchia_ordering %in% private$SUPPORTED_VECCHIA_ORDERING)) {
-          stop("GPModel: ", sQuote("vecchia_ordering"), " needs to be: ",
-               paste(sQuote(private$SUPPORTED_VECCHIA_ORDERING),collapse=", "))
-        }
-        
-        if (!(vecchia_pred_type %in% private$VECCHIA_PRED_TYPES)) {
-          stop("GPModel: ", sQuote("vecchia_pred_type"), " needs to be: ",
-               paste(sQuote(private$VECCHIA_PRED_TYPES),collapse=", "))
-        }
-        
-        if (!(cov_function %in% private$SUPPORTED_COV_FUNCTIONS)) {
-          stop("GPModel: ", sQuote("cov_function"), " needs to be: ",
-               paste(sQuote(private$SUPPORTED_COV_FUNCTIONS),collapse=", "))
-        }
-        
-        if (cov_function == "powered_exponential") {
-          if (cov_fct_shape <= 0 || cov_fct_shape > 2) {
-            stop(sQuote("cov_fct_shape"), " needs to be larger than 0 and smaller or equal than 2 for ", sQuote("cov_function=powered_exponential"))
-          }
-        }
-        
-        if (cov_function == "matern") {
-          if (!(cov_fct_shape == 0.5 || cov_fct_shape == 1.5 || cov_fct_shape == 2.5)) {
-            stop(sQuote("cov_fct_shape"), " needs to be 0.5, 1.5, or 2.5 for ", sQuote("cov_function=matern"))
-          }
-        }
-        
         private$num_gp <- 1L
         private$dim_coords <- as.integer(dim(gp_coords)[2])
         private$gp_coords <- gp_coords
         gp_coords <- as.vector(matrix(private$gp_coords)) #convert to correct format for sending to C
         private$cov_function <- cov_function
         private$cov_fct_shape <- as.numeric(cov_fct_shape)
+        private$cov_fct_taper_range <- as.numeric(cov_fct_taper_range)
         private$num_neighbors <- as.integer(num_neighbors)
         private$vecchia_ordering <- vecchia_ordering
-        vecchia_ordering <- gpb.c_str(vecchia_ordering)
         private$vecchia_pred_type <- vecchia_pred_type
         private$num_neighbors_pred <- as.integer(num_neighbors_pred)
-        private$cov_par_names <- c(private$cov_par_names,"GP_var","GP_range")
+        if (private$cov_function == "wendland") {
+          private$cov_par_names <- c(private$cov_par_names,"GP_var")
+        } else {
+          private$cov_par_names <- c(private$cov_par_names,"GP_var","GP_range")
+        }
         
         # Set data for GP random coef
         if (!is.null(gp_rand_coef_data)) {
@@ -392,11 +379,24 @@ gpb.GPModel <- R6::R6Class(
           gp_rand_coef_data <- as.vector(matrix(private$gp_rand_coef_data)) #convert to correct format for sending to C
           for (ii in 1:private$num_gp_rand_coef) {
             if (is.null(colnames(private$gp_rand_coef_data))) {
-              private$cov_par_names <- c(private$cov_par_names,paste0("GP_rand_coef_nb_", ii,"_var"),paste0("GP_rand_coef_nb_", ii,"_range"))
+              if (private$cov_function == "wendland") {
+                private$cov_par_names <- c(private$cov_par_names,
+                                           paste0("GP_rand_coef_nb_", ii,"_var"))
+              } else {
+                private$cov_par_names <- c(private$cov_par_names,
+                                           paste0("GP_rand_coef_nb_", ii,"_var"),
+                                           paste0("GP_rand_coef_nb_", ii,"_range"))
+              }
+              
             } else {
-              private$cov_par_names <- c(private$cov_par_names,
-                                         paste0("GP_rand_coef_", colnames(private$gp_rand_coef_data)[ii],"_var"),
-                                         paste0("GP_rand_coef_", colnames(private$gp_rand_coef_data)[ii],"_range"))
+              if (private$cov_function == "wendland") {
+                private$cov_par_names <- c(private$cov_par_names,
+                                           paste0("GP_rand_coef_", colnames(private$gp_rand_coef_data)[ii],"_var"))
+              } else {
+                private$cov_par_names <- c(private$cov_par_names,
+                                           paste0("GP_rand_coef_", colnames(private$gp_rand_coef_data)[ii],"_var"),
+                                           paste0("GP_rand_coef_", colnames(private$gp_rand_coef_data)[ii],"_range"))
+              }
             }
           }
         }
@@ -417,7 +417,7 @@ gpb.GPModel <- R6::R6Class(
             
             private$cluster_ids_map_to_int <- structure(1:length(unique(cluster_ids)),names=c(unique(cluster_ids)))
             cluster_ids = private$cluster_ids_map_to_int[cluster_ids]
-
+            
           }
           
         } else {
@@ -431,39 +431,38 @@ gpb.GPModel <- R6::R6Class(
       }
       
       private$determine_num_cov_pars(likelihood)
-      vecchia_pred_type_c_str <- gpb.c_str(private$vecchia_pred_type)
-      cov_function_c_str <- gpb.c_str(private$cov_function)
-      likelihood_c_str <- gpb.c_str(likelihood)
       
       # Create handle
-      handle <- 0.0
+      handle <- NULL
       
       # Attempts to create a handle for the GPModel
       # try({
       
       # Store handle
-      handle <- gpb.call("GPB_CreateREModel_R",
-                         ret = handle,
-                         private$num_data,
-                         cluster_ids,
-                         group_data_c_str,
-                         private$num_group_re,
-                         group_rand_coef_data,
-                         ind_effect_group_rand_coef,
-                         private$num_group_rand_coef,
-                         private$num_gp,
-                         gp_coords,
-                         private$dim_coords,
-                         gp_rand_coef_data,
-                         private$num_gp_rand_coef,
-                         cov_function_c_str,
-                         private$cov_fct_shape,
-                         private$vecchia_approx,
-                         private$num_neighbors,
-                         vecchia_ordering,
-                         vecchia_pred_type_c_str,
-                         private$num_neighbors_pred,
-                         likelihood_c_str)
+      handle <- .Call(
+        GPB_CreateREModel_R
+        , private$num_data
+        , cluster_ids
+        , group_data_c_str
+        , private$num_group_re
+        , group_rand_coef_data
+        , ind_effect_group_rand_coef
+        , private$num_group_rand_coef
+        , private$num_gp
+        , gp_coords
+        , private$dim_coords
+        , gp_rand_coef_data
+        , private$num_gp_rand_coef
+        , private$cov_function
+        , private$cov_fct_shape
+        , private$cov_fct_taper_range
+        , private$vecchia_approx
+        , private$num_neighbors
+        , vecchia_ordering
+        , private$vecchia_pred_type
+        , private$num_neighbors_pred
+        , likelihood
+      )
       # })
       
       # Check whether the handle was created properly if it was not stopped earlier by a stop call
@@ -618,20 +617,22 @@ gpb.GPModel <- R6::R6Class(
       
       if (is.null(X)) {
         
-        gpb.call("GPB_OptimCovPar_R",
-                 ret = NULL,
-                 private$handle,
-                 y,
-                 fixed_effects)
+        .Call(
+          GPB_OptimCovPar_R
+          , private$handle
+          , y
+          , fixed_effects
+        )
         
       } else {
         
-        gpb.call("GPB_OptimLinRegrCoefCovPar_R",
-                 ret = NULL,
-                 private$handle,
-                 y,
-                 X,
-                 private$num_coef)
+        .Call(
+          GPB_OptimLinRegrCoefCovPar_R
+          , private$handle
+          , y
+          , X
+          , private$num_coef
+        )
         
       }
       
@@ -639,6 +640,8 @@ gpb.GPModel <- R6::R6Class(
         message(paste0("GPModel: Number of iterations until convergence: ", self$get_num_optim_iter()))
       }
       
+      return(invisible(self))
+      
     },
     
     # Evaluate the negative log-ligelihood
@@ -700,11 +703,13 @@ gpb.GPModel <- R6::R6Class(
       }
       
       negll = 0.
-      gpb.call("GPB_EvalNegLogLikelihood_R",
-               ret = negll,
-               private$handle,
-               y,
-               cov_pars)
+      .Call(
+        GPB_EvalNegLogLikelihood_R
+        , private$handle
+        , y
+        , cov_pars
+        , negll
+      )
       
       return(negll)
       
@@ -792,47 +797,59 @@ gpb.GPModel <- R6::R6Class(
       nesterov_schedule_version <- private$params[["nesterov_schedule_version"]]
       trace <- private$params[["trace"]]
       momentum_offset <- private$params[["momentum_offset"]]
-      convergence_criterion <- gpb.c_str(private$params[["convergence_criterion"]])
+      convergence_criterion <- private$params[["convergence_criterion"]]
       optimizer_cov_c_str <- NULL
       init_cov_pars <- NULL
       if (!is.null(params[["optimizer_cov"]])) {
-        optimizer_cov_c_str <- gpb.c_str(params[["optimizer_cov"]])
+        optimizer_cov_c_str <- params[["optimizer_cov"]]
       }
       if (!is.null(params[["init_cov_pars"]])) {
         init_cov_pars <- params[["init_cov_pars"]]
       }
       std_dev <- private$params[["std_dev"]]
       
-      gpb.call("GPB_SetOptimConfig_R",
-               ret = NULL,
-               private$handle,
-               init_cov_pars,
-               lr_cov,
-               acc_rate_cov,
-               maxit,
-               delta_rel_conv,
-               use_nesterov_acc,
-               nesterov_schedule_version,
-               trace,
-               optimizer_cov_c_str,
-               momentum_offset,
-               convergence_criterion,
-               std_dev)
+      .Call(
+        GPB_SetOptimConfig_R
+        , private$handle
+        , init_cov_pars
+        , lr_cov
+        , acc_rate_cov
+        , maxit
+        , delta_rel_conv
+        , use_nesterov_acc
+        , nesterov_schedule_version
+        , trace
+        , optimizer_cov_c_str
+        , momentum_offset
+        , convergence_criterion
+        , std_dev
+      )
       
-      return(invisible(NULL))
+      return(invisible(self))
+     
       
     },
     
     get_optim_params = function() {
+      
       params <- private$params
-      params$optimizer_cov <- gpb.call.return.str("GPB_GetOptimizerCovPars_R",
-                                                  private$handle)
-      params$optimizer_coef <- gpb.call.return.str("GPB_GetOptimizerCoef_R",
-                                                   private$handle)
+      # Get covariance parameters optimizer
+      params$optimizer_cov <- .Call(
+        GPB_GetOptimizerCovPars_R
+        , private$handle
+      )
+      # Get linear regression coefficients optimizer
+      params$optimizer_coef <- .Call(
+        GPB_GetOptimizerCoef_R
+        , private$handle
+      )
+
       init_cov_pars <- numeric(private$num_cov_pars)
-      init_cov_pars <- gpb.call("GPB_GetInitCovPar_R",
-                                ret = init_cov_pars,
-                                private$handle)
+      .Call(
+        GPB_GetInitCovPar_R
+        , private$handle
+        , init_cov_pars
+      )
       if (sum(abs(init_cov_pars - rep(-1,private$num_cov_pars))) < 1E-6) {
         params["init_cov_pars"] <- list(NULL)
       }
@@ -845,7 +862,7 @@ gpb.GPModel <- R6::R6Class(
     
     # Set configuration parameters for the optimizer
     set_optim_coef_params = function(params = list()) {
-      num_coef=NULL
+      num_coef <- NULL
       
       if (gpb.is.null.handle(private$handle)) {
         stop("GPModel: Gaussian process model has not been initialized")
@@ -896,19 +913,20 @@ gpb.GPModel <- R6::R6Class(
       acc_rate_coef <- private$params[["acc_rate_coef"]]
       optimizer_coef_c_str <- NULL
       if (!is.null(params[["optimizer_coef"]])) {
-        optimizer_coef_c_str <- gpb.c_str(params[["optimizer_coef"]])
+        optimizer_coef_c_str <- params[["optimizer_coef"]]
       }
       
-      gpb.call("GPB_SetOptimCoefConfig_R",
-               ret = NULL,
-               private$handle,
-               num_coef,
-               init_coef,
-               lr_coef,
-               acc_rate_coef,
-               optimizer_coef_c_str)
+      .Call(
+        GPB_SetOptimCoefConfig_R
+        , private$handle
+        , num_coef
+        , init_coef
+        , lr_coef
+        , acc_rate_coef
+        , optimizer_coef_c_str
+      )
       
-      return(invisible(NULL))
+      return(invisible(self))
       
     },
     
@@ -920,10 +938,12 @@ gpb.GPModel <- R6::R6Class(
         optim_pars <- numeric(private$num_cov_pars)
       }
       
-      optim_pars <- gpb.call("GPB_GetCovPar_R",
-                             ret = optim_pars,
-                             private$handle,
-                             private$params[["std_dev"]])
+      .Call(
+        GPB_GetCovPar_R
+        , private$handle
+        , private$params[["std_dev"]]
+        , optim_pars
+      )
       
       cov_pars <- optim_pars[1:private$num_cov_pars]
       names(cov_pars) <- private$cov_par_names
@@ -948,11 +968,13 @@ gpb.GPModel <- R6::R6Class(
       } else {
         optim_pars <- numeric(private$num_coef)
       }
-      
-      optim_pars <- gpb.call("GPB_GetCoef_R",
-                             ret = optim_pars,
-                             private$handle,
-                             private$params[["std_dev"]])
+
+      .Call(
+        GPB_GetCoef_R
+        , private$handle
+        , private$params[["std_dev"]]
+        , optim_pars
+      )
       
       coef <- optim_pars[1:private$num_coef]
       names(coef) <- private$coef_names
@@ -974,6 +996,7 @@ gpb.GPModel <- R6::R6Class(
                                    X_pred = NULL) {
       
       num_data_pred <- 0
+      group_data_pred_c_str <- NULL
       
       # Set data for grouped random effects
       if (!is.null(group_data_pred)) {
@@ -998,15 +1021,9 @@ gpb.GPModel <- R6::R6Class(
         
         num_data_pred <- as.integer(dim(group_data_pred)[1])
         group_data_pred <- as.vector(group_data_pred)
-        
-        if (storage.mode(group_data_pred) != "character") {
-          storage.mode(group_data_pred) <- "character"
-        }
-        group_data_pred_c_str <- c()
-        for (i in 1:length(group_data_pred)) {
-          group_data_pred_c_str <- c(group_data_pred_c_str,gpb.c_str(group_data_pred[i]))
-        }
-        group_data_pred <- group_data_pred_c_str
+        group_data_pred_unique <- unique(group_data_pred)
+        group_data_pred_unique_c_str <- lapply(group_data_pred_unique,gpb.c_str)
+        group_data_pred_c_str <- unlist(group_data_pred_unique_c_str[match(group_data_pred,group_data_pred_unique)])
         
         # Set data for random coef for grouped REs
         if (!is.null(group_rand_coef_data_pred)) {
@@ -1040,7 +1057,7 @@ gpb.GPModel <- R6::R6Class(
         
       }
       
-      # Set data for (spatial, spatio-temporal, or temporal) Gaussian process
+      # Set data for Gaussian process
       if (!is.null(gp_coords_pred)) {
         
         if (is.numeric(gp_coords_pred)) {
@@ -1184,18 +1201,19 @@ gpb.GPModel <- R6::R6Class(
       
       private$num_data_pred <- num_data_pred
       
-      gpb.call("GPB_SetPredictionData_R",
-               ret=NULL,
-               private$handle,
-               num_data_pred,
-               cluster_ids_pred,
-               group_data_pred,
-               group_rand_coef_data_pred,
-               gp_coords_pred,
-               gp_rand_coef_data_pred,
-               X_pred)
+      .Call(
+        GPB_SetPredictionData_R
+        , private$handle
+        , num_data_pred
+        , cluster_ids_pred
+        , group_data_pred_c_str
+        , group_rand_coef_data_pred
+        , gp_coords_pred
+        , gp_rand_coef_data_pred
+        , X_pred
+      )
       
-      return(invisible(NULL))
+      return(invisible(self))
       
     },
     
@@ -1234,13 +1252,9 @@ gpb.GPModel <- R6::R6Class(
       
       group_data_pred_c_str <- NULL
       if (!is.null(vecchia_pred_type)) {
-        if (!(vecchia_pred_type %in% private$VECCHIA_PRED_TYPES)) {
-          stop("predict.GPModel: ", sQuote("vecchia_pred_type"), " needs to be either: ",
-               paste(sQuote(private$VECCHIA_PRED_TYPES),collapse=", "))
-        }
         private$vecchia_pred_type <- vecchia_pred_type
       }
-      vecchia_pred_type_c_str <- gpb.c_str(private$vecchia_pred_type)
+      vecchia_pred_type_c_str <- private$vecchia_pred_type
       
       if (!is.null(num_neighbors_pred)) {
         private$num_neighbors_pred <- as.integer(num_neighbors_pred)
@@ -1512,7 +1526,7 @@ gpb.GPModel <- R6::R6Class(
                 }
               }
               cluster_ids_pred <- cluster_ids_pred_map_to_int[cluster_ids_pred]
-                
+              
             }
             
           } else {
@@ -1610,26 +1624,28 @@ gpb.GPModel <- R6::R6Class(
         preds <- numeric(num_data_pred)
       }
       
-      preds <- gpb.call("GPB_PredictREModel_R",
-                        ret=preds,
-                        private$handle,
-                        y,
-                        num_data_pred,
-                        predict_cov_mat,
-                        predict_var,
-                        predict_response,
-                        cluster_ids_pred,
-                        group_data_pred_c_str,
-                        group_rand_coef_data_pred,
-                        gp_coords_pred,
-                        gp_rand_coef_data_pred,
-                        cov_pars,
-                        X_pred,
-                        use_saved_data,
-                        vecchia_pred_type_c_str,
-                        private$num_neighbors_pred,
-                        fixed_effects,
-                        fixed_effects_pred)
+      .Call(
+        GPB_PredictREModel_R
+        , private$handle
+        , y
+        , num_data_pred
+        , predict_cov_mat
+        , predict_var
+        , predict_response
+        , cluster_ids_pred
+        , group_data_pred_c_str
+        , group_rand_coef_data_pred
+        , gp_coords_pred
+        , gp_rand_coef_data_pred
+        , cov_pars
+        , X_pred
+        , use_saved_data
+        , vecchia_pred_type_c_str
+        , private$num_neighbors_pred
+        , fixed_effects
+        , fixed_effects_pred
+        , preds
+      )
       
       pred_mean <- preds[1:num_data_pred]
       pred_cov_mat <- NA
@@ -1692,9 +1708,11 @@ gpb.GPModel <- R6::R6Class(
     get_response_data = function() {
       
       response_data <- numeric(private$num_data)
-      response_data <- gpb.call("GPB_GetResponseData_R",
-                                ret=response_data,
-                                private$handle)
+      .Call(
+        GPB_GetResponseData_R
+        , private$handle
+        , response_data
+      )
       
       return(response_data)
     },
@@ -1705,9 +1723,11 @@ gpb.GPModel <- R6::R6Class(
         stop("GPModel: Model has no covariate data for linear predictor")
       }
       covariate_data <- numeric(private$num_data * private$num_coef)
-      covariate_data <- gpb.call("GPB_GetCovariateData_R",
-                                 ret=covariate_data,
-                                 private$handle)
+      .Call(
+        GPB_GetCovariateData_R
+        , private$handle
+        , covariate_data
+      )
       covariate_data <- matrix(covariate_data,ncol=private$num_coef)
       
       return(covariate_data)
@@ -1721,6 +1741,10 @@ gpb.GPModel <- R6::R6Class(
       return(private$cov_fct_shape)
     },
     
+    get_cov_fct_taper_range = function() {
+      return(private$cov_fct_taper_range)
+    },
+    
     get_ind_effect_group_rand_coef = function() {
       return(private$ind_effect_group_rand_coef)
     },
@@ -1731,15 +1755,19 @@ gpb.GPModel <- R6::R6Class(
     
     get_num_optim_iter = function() {
       num_it <- integer(1)
-      num_it <- gpb.call("GPB_GetNumIt_R",
-                         ret = num_it,
-                         private$handle)
+      .Call(
+        GPB_GetNumIt_R
+        , private$handle
+        , num_it
+      )
       return(num_it)
     },
     
     get_likelihood_name = function() {
-      ll_name <- gpb.call.return.str("GPB_GetLikelihoodName_R",
-                                     private$handle)
+      ll_name <- .Call(
+        GPB_GetLikelihoodName_R
+        , private$handle
+      )
       return(ll_name)
     },
     
@@ -1754,13 +1782,13 @@ gpb.GPModel <- R6::R6Class(
       if (likelihood == "gaussian" && !("Error_term" %in% private$cov_par_names)){
         private$cov_par_names <- c("Error_term",private$cov_par_names)
       }
-      likelihood_c_str <- gpb.c_str(likelihood)
-      gpb.call("GPB_SetLikelihood_R",
-               ret = NULL,
-               private$handle,
-               likelihood_c_str)
+      .Call(
+        GPB_SetLikelihood_R
+        , private$handle
+        , likelihood
+      )
       
-      return(invisible(NULL))
+      return(invisible(self))
     },
     
     model_to_list = function(include_response_data=TRUE) {
@@ -1788,8 +1816,9 @@ gpb.GPModel <- R6::R6Class(
       model_list[["vecchia_ordering"]] <- private$vecchia_ordering
       model_list[["vecchia_pred_type"]] <- private$vecchia_pred_type
       model_list[["num_neighbors_pred"]] <- private$num_neighbors_pred
-      model_list[["cov_function"]] <- self$get_cov_function()
-      model_list[["cov_fct_shape"]] <- self$get_cov_fct_shape()
+      model_list[["cov_function"]] <- private$cov_function
+      model_list[["cov_fct_shape"]] <- private$cov_fct_shape
+      model_list[["cov_fct_taper_range"]] <- private$cov_fct_taper_range
       # Covariate data
       model_list[["has_covariates"]] <- private$has_covariates
       if (private$has_covariates) {
@@ -1828,8 +1857,11 @@ gpb.GPModel <- R6::R6Class(
       # Use RJSONIO R package since jsonlite and rjson omit the last digit of a double
       save_data_json <- RJSONIO::toJSON(self$model_to_list(include_response_data=TRUE), digits=17)
       write(save_data_json, file=filename)
+      
+      return(invisible(self))
+      
     }
-    
+
   ),
   
   private = list(
@@ -1850,6 +1882,7 @@ gpb.GPModel <- R6::R6Class(
     gp_rand_coef_data = NULL,
     cov_function = "exponential",
     cov_fct_shape = 0.,
+    cov_fct_taper_range = 1.,
     vecchia_approx = FALSE,
     num_neighbors = 30L,
     vecchia_ordering = "none",
@@ -1879,15 +1912,15 @@ gpb.GPModel <- R6::R6Class(
                   trace = FALSE,
                   convergence_criterion = "relative_change_in_log_likelihood",
                   std_dev = FALSE),
-    SUPPORTED_COV_FUNCTIONS = c("exponential", "gaussian", "powered_exponential", "matern"),
-    SUPPORTED_VECCHIA_ORDERING = c("none", "random"),
-    VECCHIA_PRED_TYPES = c("order_obs_first_cond_obs_only",
-                           "order_obs_first_cond_all", "order_pred_first",
-                           "latent_order_obs_first_cond_obs_only","latent_order_obs_first_cond_all"),
     
     determine_num_cov_pars = function(likelihood) {
+      if (private$cov_function == "wendland") {
+        num_par_per_GP <- 1L
+      } else {
+        num_par_per_GP <- 2L
+      }
       private$num_cov_pars <- private$num_group_re + private$num_group_rand_coef + 
-        2L * (private$num_gp + private$num_gp_rand_coef)
+        num_par_per_GP * (private$num_gp + private$num_gp_rand_coef)
       if (likelihood == "gaussian"){
         private$num_cov_pars <- private$num_cov_pars + 1L
       }
@@ -1956,6 +1989,7 @@ GPModel <- function(group_data = NULL,
                     gp_rand_coef_data = NULL,
                     cov_function = "exponential",
                     cov_fct_shape = 0,
+                    cov_fct_taper_range = 1,
                     vecchia_approx = FALSE,
                     num_neighbors = 30L,
                     vecchia_ordering = "none",
@@ -1973,6 +2007,7 @@ GPModel <- function(group_data = NULL,
                             gp_rand_coef_data = gp_rand_coef_data,
                             cov_function = cov_function,
                             cov_fct_shape = cov_fct_shape,
+                            cov_fct_taper_range = cov_fct_taper_range,
                             vecchia_approx = vecchia_approx,
                             num_neighbors = num_neighbors,
                             vecchia_ordering = vecchia_ordering,
@@ -2154,6 +2189,7 @@ fitGPModel <- function(group_data = NULL,
                        gp_rand_coef_data = NULL,
                        cov_function = "exponential",
                        cov_fct_shape = 0,
+                       cov_fct_taper_range = 1,
                        vecchia_approx = FALSE,
                        num_neighbors = 30L,
                        vecchia_ordering = "none",
@@ -2173,6 +2209,7 @@ fitGPModel <- function(group_data = NULL,
                              gp_rand_coef_data = gp_rand_coef_data,
                              cov_function = cov_function,
                              cov_fct_shape = cov_fct_shape,
+                             cov_fct_taper_range = cov_fct_taper_range,
                              vecchia_approx = vecchia_approx,
                              num_neighbors = num_neighbors,
                              vecchia_ordering = vecchia_ordering,
@@ -2226,7 +2263,6 @@ summary.GPModel <- function(object, ...){
   print(signif(cov_pars,6))
   if (object$.__enclos_env__$private$has_covariates) {
     coef <- object$get_coef()
-    cat("\n")
     message("Linear regression coefficients:")
     print(signif(coef,6))
   }
@@ -2234,6 +2270,7 @@ summary.GPModel <- function(object, ...){
     cat("\n")
     message("Note: no convergence after the maximal number of iterations")
   }
+  return(invisible(object))
 }
 
 #' Make predictions for a \code{GPModel}
@@ -2358,7 +2395,7 @@ predict.GPModel <- function(object,
 #' @author Fabio Sigrist
 #' @export
 #' 
-saveGPModel <- function(gp_model, filename){
+saveGPModel <- function(gp_model, filename) {
   
   if (!gpb.check.r6.class(gp_model, "GPModel")) {
     stop("saveGPModel: gp_model needs to be a ", sQuote("GPModel"))
@@ -2413,3 +2450,86 @@ loadGPModel <- function(filename){
   return(invisible(gpb.GPModel$new(modelfile = filename)))
   
 }
+
+#' Generic 'set_prediction_data' method for a \code{GPModel}
+#'
+#' Generic 'set_prediction_data' method for a \code{GPModel}
+#' 
+#' @param gp_model A \code{GPModel}
+#' @param group_data_pred A \code{vector} or \code{matrix} with labels of group levels for which predictions are made (if there are grouped random effects in the \code{GPModel})
+#' @param group_rand_coef_data_pred A \code{vector} or \code{matrix} with covariate data for grouped random coefficients (if there are some in the \code{GPModel})
+#' @param gp_coords_pred A \code{matrix} with prediction coordinates (features) for Gaussian process (if there is a GP in the \code{GPModel})
+#' @param gp_rand_coef_data_pred A \code{vector} or \code{matrix} with covariate data for Gaussian process random coefficients (if there are some in the \code{GPModel})
+#' @param cluster_ids_pred A \code{vector} with IDs / labels indicating the realizations of random effects / Gaussian processes for which predictions are made (set to NULL if you have not specified this when creating the \code{GPModel})
+#' @param X_pred A \code{matrix} with covariate data for the linear regression term (if there is one in the \code{GPModel})
+#'
+#' @examples
+#' \donttest{
+#' library(gpboost)
+#' data(GPBoost_data, package = "gpboost")
+#' set.seed(1)
+#' train_ind <- sample.int(length(y),size=250)
+#' gp_model <- GPModel(group_data = group_data[train_ind,1], likelihood="gaussian")
+#' set_prediction_data(gp_model, group_data_pred = group_data[-train_ind,1])
+#' }
+#' 
+#' @author Fabio Sigrist
+#' @export 
+#' 
+set_prediction_data <- function(gp_model,
+                                group_data_pred = NULL,
+                                group_rand_coef_data_pred = NULL,
+                                gp_coords_pred = NULL,
+                                gp_rand_coef_data_pred = NULL,
+                                cluster_ids_pred = NULL,
+                                X_pred = NULL) UseMethod("set_prediction_data")
+
+#' Set prediction data for a \code{GPModel}
+#' 
+#' Set the data required for making predictions with a \code{GPModel} 
+#' 
+#' @param gp_model A \code{GPModel}
+#' @param group_data_pred A \code{vector} or \code{matrix} with labels of group levels for which predictions are made (if there are grouped random effects in the \code{GPModel})
+#' @param group_rand_coef_data_pred A \code{vector} or \code{matrix} with covariate data for grouped random coefficients (if there are some in the \code{GPModel})
+#' @param gp_coords_pred A \code{matrix} with prediction coordinates (features) for Gaussian process (if there is a GP in the \code{GPModel})
+#' @param gp_rand_coef_data_pred A \code{vector} or \code{matrix} with covariate data for Gaussian process random coefficients (if there are some in the \code{GPModel})
+#' @param cluster_ids_pred A \code{vector} with IDs / labels indicating the realizations of random effects / Gaussian processes for which predictions are made (set to NULL if you have not specified this when creating the \code{GPModel})
+#' @param X_pred A \code{matrix} with covariate data for the linear regression term (if there is one in the \code{GPModel})
+#'
+#' @return A \code{GPModel}
+#'
+#' @examples
+#' \donttest{
+#' library(gpboost)
+#' data(GPBoost_data, package = "gpboost")
+#' set.seed(1)
+#' train_ind <- sample.int(length(y),size=250)
+#' gp_model <- GPModel(group_data = group_data[train_ind,1], likelihood="gaussian")
+#' set_prediction_data(gp_model, group_data_pred = group_data[-train_ind,1])
+#' }
+#' @method set_prediction_data GPModel 
+#' @rdname set_prediction_data.GPModel
+#' @author Fabio Sigrist
+#' @export 
+#' 
+set_prediction_data.GPModel <- function(gp_model,
+                                        group_data_pred = NULL,
+                                        group_rand_coef_data_pred = NULL,
+                                        gp_coords_pred = NULL,
+                                        gp_rand_coef_data_pred = NULL,
+                                        cluster_ids_pred = NULL,
+                                        X_pred = NULL) {
+  
+  if (!gpb.check.r6.class(gp_model, "GPModel")) {
+    stop("set_prediction_data.GPModel: gp_model needs to be a ", sQuote("GPModel"))
+  }
+  
+  invisible(gp_model$set_prediction_data(group_data_pred = group_data_pred,
+                                         group_rand_coef_data_pred = group_rand_coef_data_pred,
+                                         gp_coords_pred = gp_coords_pred,
+                                         gp_rand_coef_data_pred = gp_rand_coef_data_pred,
+                                         cluster_ids_pred = cluster_ids_pred,
+                                         X_pred = X_pred))
+}
+
+
