@@ -13,7 +13,10 @@ Dataset <- R6::R6Class(
       if (!gpb.is.null.handle(x = private$handle)) {
 
         # Freeing up handle
-        gpb.call(fun_name = "LGBM_DatasetFree_R", ret = NULL, private$handle)
+        .Call(
+          LGBM_DatasetFree_R
+          , private$handle
+        )
         private$handle <- NULL
 
       }
@@ -192,7 +195,7 @@ Dataset <- R6::R6Class(
       if (!is.null(private$reference)) {
         ref_handle <- private$reference$.__enclos_env__$private$get_handle()
       }
-      handle <- gpb.null.handle()
+      handle <- NULL
 
       # Not subsetting
       if (is.null(private$used_indices)) {
@@ -200,10 +203,9 @@ Dataset <- R6::R6Class(
         # Are we using a data file?
         if (is.character(private$raw_data)) {
 
-          handle <- gpb.call(
-            fun_name = "LGBM_DatasetCreateFromFile_R"
-            , ret = handle
-            , gpb.c_str(x = private$raw_data)
+          handle <- .Call(
+            LGBM_DatasetCreateFromFile_R
+            , private$raw_data
             , params_str
             , ref_handle
           )
@@ -212,9 +214,8 @@ Dataset <- R6::R6Class(
         } else if (is.matrix(private$raw_data)) {
 
           # Are we using a matrix?
-          handle <- gpb.call(
-            fun_name = "LGBM_DatasetCreateFromMat_R"
-            , ret = handle
+          handle <- .Call(
+            LGBM_DatasetCreateFromMat_R
             , private$raw_data
             , nrow(private$raw_data)
             , ncol(private$raw_data)
@@ -227,9 +228,8 @@ Dataset <- R6::R6Class(
             stop("Cannot support large CSC matrix")
           }
           # Are we using a dgCMatrix (sparsed matrix column compressed)
-          handle <- gpb.call(
-            fun_name = "LGBM_DatasetCreateFromCSC_R"
-            , ret = handle
+          handle <- .Call(
+            LGBM_DatasetCreateFromCSC_R
             , private$raw_data@p
             , private$raw_data@i
             , private$raw_data@x
@@ -258,9 +258,8 @@ Dataset <- R6::R6Class(
         }
 
         # Construct subset
-        handle <- gpb.call(
-          fun_name = "LGBM_DatasetGetSubset_R"
-          , ret = handle
+        handle <- .Call(
+          LGBM_DatasetGetSubset_R
           , ref_handle
           , c(private$used_indices) # Adding c() fixes issue in R v3.5
           , length(private$used_indices)
@@ -333,20 +332,17 @@ Dataset <- R6::R6Class(
         num_col <- 0L
 
         # Get numeric data and numeric features
-        return(
-          c(
-            gpb.call(
-              fun_name = "LGBM_DatasetGetNumData_R"
-              , ret = num_row
-              , private$handle
-            ),
-            gpb.call(
-              fun_name = "LGBM_DatasetGetNumFeature_R"
-              , ret = num_col
-              , private$handle
-            )
-          )
+        .Call(
+          LGBM_DatasetGetNumData_R
+          , private$handle
+          , num_row
         )
+        .Call(
+          LGBM_DatasetGetNumFeature_R
+          , private$handle
+          , num_col
+        )
+        return(c(num_row, num_col))
 
       } else if (is.matrix(private$raw_data) || methods::is(private$raw_data, "dgCMatrix")) {
 
@@ -373,11 +369,10 @@ Dataset <- R6::R6Class(
       if (!gpb.is.null.handle(x = private$handle)) {
 
         # Get feature names and write them
-        cnames <- gpb.call.return.str(
-            fun_name = "LGBM_DatasetGetFeatureNames_R"
-            , private$handle
+        private$colnames <- .Call(
+          LGBM_DatasetGetFeatureNames_R
+          , private$handle
         )
-        private$colnames <- as.character(base::strsplit(cnames, "\t")[[1L]])
         return(private$colnames)
 
       } else if (is.matrix(private$raw_data) || methods::is(private$raw_data, "dgCMatrix")) {
@@ -416,11 +411,10 @@ Dataset <- R6::R6Class(
 
         # Merge names with tab separation
         merged_name <- paste0(as.list(private$colnames), collapse = "\t")
-        gpb.call(
-          fun_name = "LGBM_DatasetSetFeatureNames_R"
-          , ret = NULL
+        .Call(
+          LGBM_DatasetSetFeatureNames_R
           , private$handle
-          , gpb.c_str(x = merged_name)
+          , merged_name
         )
 
       }
@@ -449,11 +443,11 @@ Dataset <- R6::R6Class(
 
         # Get field size of info
         info_len <- 0L
-        info_len <- gpb.call(
-          fun_name = "LGBM_DatasetGetFieldSize_R"
-          , ret = info_len
+        .Call(
+          LGBM_DatasetGetFieldSize_R
           , private$handle
-          , gpb.c_str(x = name)
+          , name
+          , info_len
         )
 
         # Check if info is not empty
@@ -467,11 +461,11 @@ Dataset <- R6::R6Class(
             numeric(info_len) # Numeric
           }
 
-          ret <- gpb.call(
-            fun_name = "LGBM_DatasetGetField_R"
-            , ret = ret
+          .Call(
+            LGBM_DatasetGetField_R
             , private$handle
-            , gpb.c_str(x = name)
+            , name
+            , ret
           )
 
           private$info[[name]] <- ret
@@ -508,11 +502,10 @@ Dataset <- R6::R6Class(
 
         if (length(info) > 0L) {
 
-          gpb.call(
-            fun_name = "LGBM_DatasetSetField_R"
-            , ret = NULL
+          .Call(
+            LGBM_DatasetSetField_R
             , private$handle
-            , gpb.c_str(x = name)
+            , name
             , info
             , length(info)
           )
@@ -548,7 +541,9 @@ Dataset <- R6::R6Class(
 
     },
 
-    # Update parameters
+    # [description] Update Dataset parameters. If it has not been constructed yet,
+    #               this operation just happens on the R side (updating private$params).
+    #               If it has been constructed, parameters will be updated on the C++ side
     update_params = function(params) {
       if (length(params) == 0L) {
         return(invisible(self))
@@ -556,26 +551,24 @@ Dataset <- R6::R6Class(
       if (gpb.is.null.handle(x = private$handle)) {
         private$params <- modifyList(private$params, params)
       } else {
-        call_state <- 0L
-        call_state <- .Call(
-          "LGBM_DatasetUpdateParamChecking_R"
-          , gpb.params2str(params = private$params)
-          , gpb.params2str(params = params)
-          , call_state
-          , PACKAGE = "gpboost"
-        )
-        call_state <- as.integer(call_state)
-        if (call_state != 0L) {
-
-          # raise error if raw data is freed
+        tryCatch({
+          .Call(
+            LGBM_DatasetUpdateParamChecking_R
+            , gpb.params2str(params = private$params)
+            , gpb.params2str(params = params)
+          )
+        }, error = function(e) {
+          # If updating failed but raw data is not available, raise an error because
+          # achieving what the user asked for is not possible
           if (is.null(private$raw_data)) {
-            gpb.last_error()
+            stop(e)
           }
-
-          # Overwrite paramms
+          
+          # If updating failed but raw data is available, modify the params
+          # on the R side and re-set ("deconstruct") the Dataset
           private$params <- modifyList(private$params, params)
           self$finalize()
-        }
+        })
       }
       return(invisible(self))
 
@@ -660,11 +653,10 @@ Dataset <- R6::R6Class(
 
       # Store binary data
       self$construct()
-      gpb.call(
-        fun_name = "LGBM_DatasetSaveBinary_R"
-        , ret = NULL
+      .Call(
+        LGBM_DatasetSaveBinary_R
         , private$handle
-        , gpb.c_str(x = fname)
+        , fname
       )
       return(invisible(self))
     }
@@ -733,11 +725,20 @@ Dataset <- R6::R6Class(
 #' @description Construct \code{gpb.Dataset} object from dense matrix, sparse matrix
 #'              or local file (that was created previously by saving an \code{gpb.Dataset}).
 #' @param data a \code{matrix} object, a \code{dgCMatrix} object or a character representing a filename
-#' @param params a list of parameters
-#' @param reference reference dataset
+#' @param params a list of parameters. See
+#'               \href{https://github.com/fabsig/GPBoost/blob/master/docs/Parameters.rst#dataset-parameters}{
+#'               the "Dataset Parameters" section of the parameter documentation} for a list of parameters
+#'               and valid values.
+#' @param reference reference dataset. When GPBoost creates a Dataset, it does some preprocessing like binning
+#'                  continuous features into histograms. If you want to apply the same bin boundaries from an existing
+#'                  dataset to new \code{data}, pass that existing Dataset to this argument.
 #' @param colnames names of columns
-#' @param categorical_feature categorical features
-#' @param free_raw_data TRUE for need to free raw data after construct
+#' @param categorical_feature categorical features. This can either be a character vector of feature
+#'                            names or an integer vector with the indices of the features (e.g.
+#'                            \code{c(1L, 10L)} to say "the first and tenth columns").
+#' @param free_raw_data GPBoost constructs its data format, called a "Dataset", from tabular data.
+#'                      By default, this Dataset object on the R side does keep a copy of the raw data.
+#'                      If you set \code{free_raw_data = TRUE}, no copy of the raw data is kept (this reduces memory usage)
 #' @param info a list of information of the \code{gpb.Dataset} object
 #' @param ... other information to pass to \code{info} or parameters pass to \code{params}
 #'
