@@ -1,3 +1,6 @@
+## Examples used in the R package documentation / help files
+## Author: Fabio Sigrist
+
 library(gpboost)
 data(GPBoost_data, package = "gpboost")
 
@@ -283,6 +286,38 @@ bst <- gpb.train(data = dtrain,
 summary(gp_model)
 
 
+#--------------------Parameter tuning-------------------------
+# Create random effects model, dataset, and define parameter grif
+gp_model <- GPModel(group_data = group_data[,1], likelihood="gaussian")
+dtrain <- gpb.Dataset(X, label = y)
+params <- list(objective = "regression_l2")
+param_grid = list("learning_rate" = c(0.1,0.01), "min_data_in_leaf" = c(20),
+                  "max_depth" = c(5,10), "num_leaves" = 2^17, "max_bin" = c(255,1000))
+# Parameter tuning using cross-validation and deterministic grid search
+set.seed(1)
+opt_params <- gpb.grid.search.tune.parameters(param_grid = param_grid,
+                                              params = params,
+                                              num_try_random = NULL,
+                                              nfold = 4,
+                                              data = dtrain,
+                                              gp_model = gp_model,
+                                              verbose_eval = 1,
+                                              nrounds = 1000,
+                                              early_stopping_rounds = 5,
+                                              eval = "l2")
+# Parameter tuning using cross-validation and random grid search
+set.seed(1)
+opt_params <- gpb.grid.search.tune.parameters(param_grid = param_grid,
+                                              params = params,
+                                              num_try_random = 4,
+                                              nfold = 4,
+                                              data = dtrain,
+                                              gp_model = gp_model,
+                                              verbose_eval = 1,
+                                              nrounds = 1000,
+                                              early_stopping_rounds = 5,
+                                              eval = "l2")
+
 #--------------------Cross validation-------------------------
 # Create random effects model and dataset
 gp_model <- GPModel(group_data = group_data[,1], likelihood="gaussian")
