@@ -7,7 +7,46 @@
 #define GPBOOST_R_H_
 
 #include <LightGBM/c_api.h>
-#include "R_object_helper.h"
+
+#define R_NO_REMAP
+#define R_USE_C99_IN_CXX
+#include <Rinternals.h>
+
+inline double* R_REAL_PTR(SEXP x) {
+	if (Rf_isNull(x)) {
+		return nullptr;
+	}
+	else {
+		return REAL(x);
+	}
+}
+
+inline int* R_INT_PTR(SEXP x) {
+	if (Rf_isNull(x)) {
+		return nullptr;
+	}
+	else {
+		return INTEGER(x);
+	}
+}
+
+inline const char* R_CHAR_PTR_FROM_RAW(SEXP x) {
+	if (Rf_isNull(x)) {
+		return nullptr;
+	}
+	else {
+		return (reinterpret_cast<const char*>(RAW(x)));
+	}
+}
+
+inline const char* R_CHAR_PTR(SEXP x) {
+	if (Rf_isNull(x)) {
+		return nullptr;
+	}
+	else {
+		return CHAR(Rf_asChar(x));
+	}
+}
 
 /*!
 * \brief get string message of the last error
@@ -16,10 +55,15 @@
 * \return err_msg error information
 * \return error information
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_GetLastError_R(
-	LGBM_SE buf_len,
-	LGBM_SE actual_len,
-	LGBM_SE err_msg
+LIGHTGBM_C_EXPORT SEXP LGBM_GetLastError_R();
+
+/*!
+* \brief check if an R external pointer (like a Booster or Dataset handle) is a null pointer
+* \param handle handle for a Booster, Dataset, or Predictor
+* \return R logical, TRUE if the handle is a null pointer
+*/
+LIGHTGBM_C_EXPORT SEXP LGBM_HandleIsNull_R(
+	SEXP handle
 );
 
 // --- start Dataset interface
@@ -29,15 +73,12 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_GetLastError_R(
 * \param filename the name of the file
 * \param parameters additional parameters
 * \param reference used to align bin mapper with other dataset, nullptr means not used
-* \param out created dataset
-* \return 0 when succeed, -1 when failure happens
+* \return Dataset handle
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetCreateFromFile_R(
-	LGBM_SE filename,
-	LGBM_SE parameters,
-	LGBM_SE reference,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetCreateFromFile_R(
+	SEXP filename,
+	SEXP parameters,
+	SEXP reference
 );
 
 /*!
@@ -45,45 +86,39 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetCreateFromFile_R(
 * \param indptr pointer to row headers
 * \param indices findex
 * \param data fvalue
-* \param nindptr number of cols in the matrix + 1
+* \param num_indptr number of cols in the matrix + 1
 * \param nelem number of nonzero elements in the matrix
 * \param num_row number of rows
 * \param parameters additional parameters
 * \param reference used to align bin mapper with other dataset, nullptr means not used
-* \param out created dataset
-* \return 0 when succeed, -1 when failure happens
+* \return Dataset handle
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetCreateFromCSC_R(
-	LGBM_SE indptr,
-	LGBM_SE indices,
-	LGBM_SE data,
-	LGBM_SE nindptr,
-	LGBM_SE nelem,
-	LGBM_SE num_row,
-	LGBM_SE parameters,
-	LGBM_SE reference,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetCreateFromCSC_R(
+	SEXP indptr,
+	SEXP indices,
+	SEXP data,
+	SEXP num_indptr,
+	SEXP nelem,
+	SEXP num_row,
+	SEXP parameters,
+	SEXP reference
 );
 
 /*!
 * \brief create dataset from dense matrix
 * \param data matric data
-* \param nrow number of rows
+* \param num_row number of rows
 * \param ncol number columns
 * \param parameters additional parameters
 * \param reference used to align bin mapper with other dataset, nullptr means not used
-* \param out created dataset
-* \return 0 when succeed, -1 when failure happens
+* \return Dataset handle
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetCreateFromMat_R(
-	LGBM_SE data,
-	LGBM_SE nrow,
-	LGBM_SE ncol,
-	LGBM_SE parameters,
-	LGBM_SE reference,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetCreateFromMat_R(
+	SEXP data,
+	SEXP num_row,
+	SEXP ncol,
+	SEXP parameters,
+	SEXP reference
 );
 
 /*!
@@ -92,42 +127,33 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetCreateFromMat_R(
 * \param used_row_indices Indices used in subset
 * \param len_used_row_indices length of Indices used in subset
 * \param parameters additional parameters
-* \param out created dataset
-* \return 0 when succeed, -1 when failure happens
+* \return Dataset handle
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetGetSubset_R(
-	LGBM_SE handle,
-	LGBM_SE used_row_indices,
-	LGBM_SE len_used_row_indices,
-	LGBM_SE parameters,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetGetSubset_R(
+	SEXP handle,
+	SEXP used_row_indices,
+	SEXP len_used_row_indices,
+	SEXP parameters
 );
 
 /*!
 * \brief save feature names to Dataset
 * \param handle handle
 * \param feature_names feature names
-* \return 0 when succeed, -1 when failure happens
+* \return R character vector of feature names
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetSetFeatureNames_R(
-	LGBM_SE handle,
-	LGBM_SE feature_names,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetSetFeatureNames_R(
+	SEXP handle,
+	SEXP feature_names
 );
 
 /*!
-* \brief save feature names to Dataset
-* \param handle handle
-* \param feature_names feature names
-* \return 0 when succeed, -1 when failure happens
+* \brief get feature names from Dataset
+* \param handle Dataset handle
+* \return an R character vector with feature names from the Dataset or NULL if no feature names
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetGetFeatureNames_R(
-	LGBM_SE handle,
-	LGBM_SE buf_len,
-	LGBM_SE actual_len,
-	LGBM_SE feature_names,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetGetFeatureNames_R(
+	SEXP handle
 );
 
 /*!
@@ -136,10 +162,9 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetGetFeatureNames_R(
 * \param filename file name
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetSaveBinary_R(
-	LGBM_SE handle,
-	LGBM_SE filename,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetSaveBinary_R(
+	SEXP handle,
+	SEXP filename
 );
 
 /*!
@@ -147,9 +172,8 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetSaveBinary_R(
 * \param handle an instance of dataset
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetFree_R(
-	LGBM_SE handle,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetFree_R(
+	SEXP handle
 );
 
 /*!
@@ -162,12 +186,11 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetFree_R(
 * \param num_element number of element in field_data
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetSetField_R(
-	LGBM_SE handle,
-	LGBM_SE field_name,
-	LGBM_SE field_data,
-	LGBM_SE num_element,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetSetField_R(
+	SEXP handle,
+	SEXP field_name,
+	SEXP field_data,
+	SEXP num_element
 );
 
 /*!
@@ -177,11 +200,10 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetSetField_R(
 * \param out size of info vector from dataset
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetGetFieldSize_R(
-	LGBM_SE handle,
-	LGBM_SE field_name,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetGetFieldSize_R(
+	SEXP handle,
+	SEXP field_name,
+	SEXP out
 );
 
 /*!
@@ -191,11 +213,10 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetGetFieldSize_R(
 * \param field_data pointer to vector
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetGetField_R(
-	LGBM_SE handle,
-	LGBM_SE field_name,
-	LGBM_SE field_data,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetGetField_R(
+	SEXP handle,
+	SEXP field_name,
+	SEXP field_data
 );
 
 /*!
@@ -204,10 +225,9 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetGetField_R(
  * \param new_params New dataset parameters
  * \return 0 when succeed, -1 when failure happens
  */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetUpdateParamChecking_R(
-	LGBM_SE old_params,
-	LGBM_SE new_params,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetUpdateParamChecking_R(
+	SEXP old_params,
+	SEXP new_params
 );
 
 /*!
@@ -216,10 +236,9 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetUpdateParamChecking_R(
 * \param out The address to hold number of data
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetGetNumData_R(
-	LGBM_SE handle,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetGetNumData_R(
+	SEXP handle,
+	SEXP out
 );
 
 /*!
@@ -228,10 +247,9 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetGetNumData_R(
 * \param out The output of number of features
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetGetNumFeature_R(
-	LGBM_SE handle,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_DatasetGetNumFeature_R(
+	SEXP handle,
+	SEXP out
 );
 
 // --- start Booster interfaces
@@ -240,14 +258,11 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_DatasetGetNumFeature_R(
 * \brief create a new boosting learner
 * \param train_data training data set
 * \param parameters format: 'key1=value1 key2=value2'
-* \param out handle of created Booster
-* \return 0 when succeed, -1 when failure happens
+* \return Booster handle
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterCreate_R(
-	LGBM_SE train_data,
-	LGBM_SE parameters,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterCreate_R(
+	SEXP train_data,
+	SEXP parameters
 );
 
 /*!
@@ -255,48 +270,39 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterCreate_R(
 * \param train_data training data set
 * \param parameters format: 'key1=value1 key2=value2'
 * \param re_model Gaussian process model
-* \param out handle of created Booster
-* \return 0 when succeed, -1 when failure happens
+* \return Booster handle
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_GPBoosterCreate_R(
-	LGBM_SE train_data,
-	LGBM_SE parameters,
-	LGBM_SE re_model,
-	LGBM_SE out,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP LGBM_GPBoosterCreate_R(
+	SEXP train_data,
+	SEXP parameters,
+	SEXP re_model
+);
 
 /*!
 * \brief free obj in handle
 * \param handle handle to be freed
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterFree_R(
-	LGBM_SE handle,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterFree_R(
+	SEXP handle
 );
 
 /*!
 * \brief load an existing boosting from model file
 * \param filename filename of model
-* \param out handle of created Booster
-* \return 0 when succeed, -1 when failure happens
+* \return Booster handle
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterCreateFromModelfile_R(
-	LGBM_SE filename,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterCreateFromModelfile_R(
+	SEXP filename
 );
 
 /*!
 * \brief load an existing boosting from model_str
 * \param model_str string containing the model
-* \param out handle of created Booster
-* \return 0 when succeed, -1 when failure happens
+* \return Booster handle
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterLoadModelFromString_R(
-	LGBM_SE model_str,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterLoadModelFromString_R(
+	SEXP model_str
 );
 
 /*!
@@ -305,10 +311,9 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterLoadModelFromString_R(
 * \param other_handle
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterMerge_R(
-	LGBM_SE handle,
-	LGBM_SE other_handle,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterMerge_R(
+	SEXP handle,
+	SEXP other_handle
 );
 
 /*!
@@ -317,10 +322,9 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterMerge_R(
 * \param valid_data validation data set
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterAddValidData_R(
-	LGBM_SE handle,
-	LGBM_SE valid_data,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterAddValidData_R(
+	SEXP handle,
+	SEXP valid_data
 );
 
 /*!
@@ -329,10 +333,9 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterAddValidData_R(
 * \param train_data training data set
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterResetTrainingData_R(
-	LGBM_SE handle,
-	LGBM_SE train_data,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterResetTrainingData_R(
+	SEXP handle,
+	SEXP train_data
 );
 
 /*!
@@ -341,10 +344,9 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterResetTrainingData_R(
 * \param parameters format: 'key1=value1 key2=value2'
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterResetParameter_R(
-	LGBM_SE handle,
-	LGBM_SE parameters,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterResetParameter_R(
+	SEXP handle,
+	SEXP parameters
 );
 
 /*!
@@ -353,10 +355,9 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterResetParameter_R(
 * \param out number of classes
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetNumClasses_R(
-	LGBM_SE handle,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterGetNumClasses_R(
+	SEXP handle,
+	SEXP out
 );
 
 /*!
@@ -364,9 +365,8 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetNumClasses_R(
 * \param handle handle
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterUpdateOneIter_R(
-	LGBM_SE handle,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterUpdateOneIter_R(
+	SEXP handle
 );
 
 /*!
@@ -378,12 +378,11 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterUpdateOneIter_R(
 * \param len length of grad/hess
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterUpdateOneIterCustom_R(
-	LGBM_SE handle,
-	LGBM_SE grad,
-	LGBM_SE hess,
-	LGBM_SE len,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterUpdateOneIterCustom_R(
+	SEXP handle,
+	SEXP grad,
+	SEXP hess,
+	SEXP len
 );
 
 /*!
@@ -391,9 +390,8 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterUpdateOneIterCustom_R(
 * \param handle handle
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterRollbackOneIter_R(
-	LGBM_SE handle,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterRollbackOneIter_R(
+	SEXP handle
 );
 
 /*!
@@ -401,10 +399,9 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterRollbackOneIter_R(
 * \param out iteration of boosting rounds
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetCurrentIteration_R(
-	LGBM_SE handle,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterGetCurrentIteration_R(
+	SEXP handle,
+	SEXP out
 );
 
 /*!
@@ -413,10 +410,9 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetCurrentIteration_R(
 * \param[out] out_results Result pointing to max value
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetUpperBoundValue_R(
-	LGBM_SE handle,
-	LGBM_SE out_result,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterGetUpperBoundValue_R(
+	SEXP handle,
+	SEXP out_result
 );
 
 /*!
@@ -425,23 +421,18 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetUpperBoundValue_R(
 * \param[out] out_results Result pointing to min value
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetLowerBoundValue_R(
-	LGBM_SE handle,
-	LGBM_SE out_result,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterGetLowerBoundValue_R(
+	SEXP handle,
+	SEXP out_result
 );
 
 /*!
-* \brief Get Name of eval
-* \param eval_names eval names
-* \return 0 when succeed, -1 when failure happens
+* \brief Get names of eval metrics
+* \param handle Handle of booster
+* \return R character vector with names of eval metrics
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetEvalNames_R(
-	LGBM_SE handle,
-	LGBM_SE buf_len,
-	LGBM_SE actual_len,
-	LGBM_SE eval_names,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterGetEvalNames_R(
+	SEXP handle
 );
 
 /*!
@@ -451,11 +442,10 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetEvalNames_R(
 * \param out_result float array contains result
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetEval_R(
-	LGBM_SE handle,
-	LGBM_SE data_idx,
-	LGBM_SE out_result,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterGetEval_R(
+	SEXP handle,
+	SEXP data_idx,
+	SEXP out_result
 );
 
 /*!
@@ -465,11 +455,10 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetEval_R(
 * \param out size of predict
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetNumPredict_R(
-	LGBM_SE handle,
-	LGBM_SE data_idx,
-	LGBM_SE out,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterGetNumPredict_R(
+	SEXP handle,
+	SEXP data_idx,
+	SEXP out
 );
 
 /*!
@@ -480,11 +469,10 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetNumPredict_R(
 * \param out_result, used to store predict result, should pre-allocate memory
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetPredict_R(
-	LGBM_SE handle,
-	LGBM_SE data_idx,
-	LGBM_SE out_result,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterGetPredict_R(
+	SEXP handle,
+	SEXP data_idx,
+	SEXP out_result
 );
 
 /*!
@@ -498,18 +486,17 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterGetPredict_R(
 * \return 0 when succeed, -1 when failure happens
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterPredictForFile_R(
-	LGBM_SE handle,
-	LGBM_SE data_filename,
-	LGBM_SE data_has_header,
-	LGBM_SE is_rawscore,
-	LGBM_SE is_leafidx,
-	LGBM_SE is_predcontrib,
-	LGBM_SE start_iteration,
-	LGBM_SE num_iteration,
-	LGBM_SE parameter,
-	LGBM_SE result_filename,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterPredictForFile_R(
+	SEXP handle,
+	SEXP data_filename,
+	SEXP data_has_header,
+	SEXP is_rawscore,
+	SEXP is_leafidx,
+	SEXP is_predcontrib,
+	SEXP start_iteration,
+	SEXP num_iteration,
+	SEXP parameter,
+	SEXP result_filename
 );
 
 /*!
@@ -522,16 +509,15 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterPredictForFile_R(
 * \param out_len length of prediction
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterCalcNumPredict_R(
-	LGBM_SE handle,
-	LGBM_SE num_row,
-	LGBM_SE is_rawscore,
-	LGBM_SE is_leafidx,
-	LGBM_SE is_predcontrib,
-	LGBM_SE start_iteration,
-	LGBM_SE num_iteration,
-	LGBM_SE out_len,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterCalcNumPredict_R(
+	SEXP handle,
+	SEXP num_row,
+	SEXP is_rawscore,
+	SEXP is_leafidx,
+	SEXP is_predcontrib,
+	SEXP start_iteration,
+	SEXP num_iteration,
+	SEXP out_len
 );
 
 /*!
@@ -543,7 +529,7 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterCalcNumPredict_R(
 * \param indptr pointer to row headers
 * \param indices findex
 * \param data fvalue
-* \param nindptr number of cols in the matrix + 1
+* \param num_indptr number of cols in the matrix + 1
 * \param nelem number of non-zero elements in the matrix
 * \param num_row number of rows
 * \param is_rawscore
@@ -552,22 +538,21 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterCalcNumPredict_R(
 * \param out prediction result
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterPredictForCSC_R(
-	LGBM_SE handle,
-	LGBM_SE indptr,
-	LGBM_SE indices,
-	LGBM_SE data,
-	LGBM_SE nindptr,
-	LGBM_SE nelem,
-	LGBM_SE num_row,
-	LGBM_SE is_rawscore,
-	LGBM_SE is_leafidx,
-	LGBM_SE is_predcontrib,
-	LGBM_SE start_iteration,
-	LGBM_SE num_iteration,
-	LGBM_SE parameter,
-	LGBM_SE out_result,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterPredictForCSC_R(
+	SEXP handle,
+	SEXP indptr,
+	SEXP indices,
+	SEXP data,
+	SEXP num_indptr,
+	SEXP nelem,
+	SEXP num_row,
+	SEXP is_rawscore,
+	SEXP is_leafidx,
+	SEXP is_predcontrib,
+	SEXP start_iteration,
+	SEXP num_iteration,
+	SEXP parameter,
+	SEXP out_result
 );
 
 /*!
@@ -577,7 +562,7 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterPredictForCSC_R(
 *               for leaf index, its length is equal to num_class * num_data * num_iteration
 * \param handle handle
 * \param data pointer to the data space
-* \param nrow number of rows
+* \param num_row number of rows
 * \param ncol number columns
 * \param is_rawscore
 * \param is_leafidx
@@ -585,19 +570,18 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterPredictForCSC_R(
 * \param out prediction result
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterPredictForMat_R(
-	LGBM_SE handle,
-	LGBM_SE data,
-	LGBM_SE nrow,
-	LGBM_SE ncol,
-	LGBM_SE is_rawscore,
-	LGBM_SE is_leafidx,
-	LGBM_SE is_predcontrib,
-	LGBM_SE start_iteration,
-	LGBM_SE num_iteration,
-	LGBM_SE parameter,
-	LGBM_SE out_result,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterPredictForMat_R(
+	SEXP handle,
+	SEXP data,
+	SEXP num_row,
+	SEXP ncol,
+	SEXP is_rawscore,
+	SEXP is_leafidx,
+	SEXP is_predcontrib,
+	SEXP start_iteration,
+	SEXP num_iteration,
+	SEXP parameter,
+	SEXP out_result
 );
 
 /*!
@@ -607,48 +591,39 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterPredictForMat_R(
 * \param filename file name
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterSaveModel_R(
-	LGBM_SE handle,
-	LGBM_SE num_iteration,
-	LGBM_SE feature_importance_type,
-	LGBM_SE filename,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterSaveModel_R(
+	SEXP handle,
+	SEXP num_iteration,
+	SEXP feature_importance_type,
+	SEXP filename
 );
 
 /*!
 * \brief create string containing model
-* \param handle handle
+* \param handle Booster handle
 * \param start_iteration Start index of the iteration that should be saved
 * \param num_iteration, <= 0 means save all
-* \param out_str string of model
-* \return 0 when succeed, -1 when failure happens
+* \param feature_importance_type type of feature importance, 0: split, 1: gain
+* \return R character vector (length=1) with model string
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterSaveModelToString_R(
-	LGBM_SE handle,
-	LGBM_SE start_iteration,
-	LGBM_SE num_iteration,
-	LGBM_SE feature_importance_type,
-	LGBM_SE buffer_len,
-	LGBM_SE actual_len,
-	LGBM_SE out_str,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterSaveModelToString_R(
+	SEXP handle,
+	SEXP start_iteration,
+	SEXP num_iteration,
+	SEXP feature_importance_type
 );
 
 /*!
-* \brief dump model to json
-* \param handle handle
+* \brief dump model to JSON
+* \param handle Booster handle
 * \param num_iteration, <= 0 means save all
-* \param out_str json format string of model
-* \return 0 when succeed, -1 when failure happens
+* \param feature_importance_type type of feature importance, 0: split, 1: gain
+* \return R character vector (length=1) with model JSON
 */
-LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterDumpModel_R(
-	LGBM_SE handle,
-	LGBM_SE num_iteration,
-	LGBM_SE feature_importance_type,
-	LGBM_SE buffer_len,
-	LGBM_SE actual_len,
-	LGBM_SE out_str,
-	LGBM_SE call_state
+LIGHTGBM_C_EXPORT SEXP LGBM_BoosterDumpModel_R(
+	SEXP handle,
+	SEXP num_iteration,
+	SEXP feature_importance_type
 );
 
 // Below here are REModel / GPModel related functions
@@ -676,42 +651,40 @@ LIGHTGBM_C_EXPORT LGBM_SE LGBM_BoosterDumpModel_R(
 * \param vecchia_pred_type Type of Vecchia approximation for making predictions. "order_obs_first_cond_obs_only" = observed data is ordered first and neighbors are only observed points, "order_obs_first_cond_all" = observed data is ordered first and neighbors are selected among all points (observed + predicted), "order_pred_first" = predicted data is ordered first for making predictions, "latent_order_obs_first_cond_obs_only"  = Vecchia approximation for the latent process and observed data is ordered first and neighbors are only observed points, "latent_order_obs_first_cond_all"  = Vecchia approximation for the latent process and observed data is ordered first and neighbors are selected among all points
 * \param num_neighbors_pred The number of neighbors used in the Vecchia approximation for making predictions
 * \param likelihood Likelihood function for the observed response variable. Default = "gaussian"
-* \param out Created REModel
-* \return 0 when succeed, -1 when failure happens
+* \return REModel handle
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_CreateREModel_R(
-	LGBM_SE ndata,
-	LGBM_SE cluster_ids_data,
-	LGBM_SE re_group_data,
-	LGBM_SE num_re_group,
-	LGBM_SE re_group_rand_coef_data,
-	LGBM_SE ind_effect_group_rand_coef,
-	LGBM_SE num_re_group_rand_coef,
-	LGBM_SE num_gp,
-	LGBM_SE gp_coords_data,
-	LGBM_SE dim_gp_coords,
-	LGBM_SE gp_rand_coef_data,
-	LGBM_SE num_gp_rand_coef,
-	LGBM_SE cov_fct,
-	LGBM_SE cov_fct_shape,
-	LGBM_SE cov_fct_taper_range,
-	LGBM_SE vecchia_approx,
-	LGBM_SE num_neighbors,
-	LGBM_SE vecchia_ordering,
-	LGBM_SE vecchia_pred_type,
-	LGBM_SE num_neighbors_pred,
-	LGBM_SE likelihood,
-	LGBM_SE out,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_CreateREModel_R(
+	SEXP ndata,
+	SEXP cluster_ids_data,
+	SEXP re_group_data,
+	SEXP num_re_group,
+	SEXP re_group_rand_coef_data,
+	SEXP ind_effect_group_rand_coef,
+	SEXP num_re_group_rand_coef,
+	SEXP num_gp,
+	SEXP gp_coords_data,
+	SEXP dim_gp_coords,
+	SEXP gp_rand_coef_data,
+	SEXP num_gp_rand_coef,
+	SEXP cov_fct,
+	SEXP cov_fct_shape,
+	SEXP cov_fct_taper_range,
+	SEXP vecchia_approx,
+	SEXP num_neighbors,
+	SEXP vecchia_ordering,
+	SEXP vecchia_pred_type,
+	SEXP num_neighbors_pred,
+	SEXP likelihood
+);
 
 /*!
 * \brief free obj in handle
 * \param handle handle of REModel
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_REModelFree_R(
-	LGBM_SE handle,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_REModelFree_R(
+	SEXP handle
+);
 
 /*!
 * \brief Set configuration parameters for the optimizer
@@ -730,21 +703,21 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_REModelFree_R(
 * \param calc_std_dev If true, asymptotic standard deviations for the MLE of the covariance parameters are calculated as the diagonal of the inverse Fisher information
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_SetOptimConfig_R(
-	LGBM_SE handle,
-	LGBM_SE init_cov_pars,
-	LGBM_SE lr,
-	LGBM_SE acc_rate_cov,
-	LGBM_SE max_iter,
-	LGBM_SE delta_rel_conv,
-	LGBM_SE use_nesterov_acc,
-	LGBM_SE nesterov_schedule_version,
-	LGBM_SE trace,
-	LGBM_SE optimizer,
-	LGBM_SE momentum_offset,
-	LGBM_SE convergence_criterion,
-	LGBM_SE calc_std_dev,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_SetOptimConfig_R(
+	SEXP handle,
+	SEXP init_cov_pars,
+	SEXP lr,
+	SEXP acc_rate_cov,
+	SEXP max_iter,
+	SEXP delta_rel_conv,
+	SEXP use_nesterov_acc,
+	SEXP nesterov_schedule_version,
+	SEXP trace,
+	SEXP optimizer,
+	SEXP momentum_offset,
+	SEXP convergence_criterion,
+	SEXP calc_std_dev
+);
 
 /*!
 * \brief Set configuration parameters for the optimizer for linear regression coefficients
@@ -756,14 +729,14 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_SetOptimConfig_R(
 * \param optimizer Options: "gradient_descent" or "wls" (coordinate descent using weighted least squares)
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_SetOptimCoefConfig_R(
-	LGBM_SE handle,
-	LGBM_SE num_covariates,
-	LGBM_SE init_coef,
-	LGBM_SE lr_coef,
-	LGBM_SE acc_rate_coef,
-	LGBM_SE optimizer,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_SetOptimCoefConfig_R(
+	SEXP handle,
+	SEXP num_covariates,
+	SEXP init_coef,
+	SEXP lr_coef,
+	SEXP acc_rate_coef,
+	SEXP optimizer
+);
 
 /*!
 * \brief Find parameters that minimize the negative log-ligelihood (=MLE)
@@ -772,11 +745,11 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_SetOptimCoefConfig_R(
 * \param fixed_effects Fixed effects component F of location parameter (only used for non-Gaussian data). For Gaussian data, this is ignored
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_OptimCovPar_R(
-	LGBM_SE handle,
-	LGBM_SE y_data,
-	LGBM_SE fixed_effects,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_OptimCovPar_R(
+	SEXP handle,
+	SEXP y_data,
+	SEXP fixed_effects
+);
 
 /*!
 * \brief Find parameters that minimize the negative log-ligelihood (=MLE)
@@ -787,12 +760,12 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_OptimCovPar_R(
 * \param num_covariates Number of covariates
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_OptimLinRegrCoefCovPar_R(
-	LGBM_SE handle,
-	LGBM_SE y_data,
-	LGBM_SE covariate_data,
-	LGBM_SE num_covariates,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_OptimLinRegrCoefCovPar_R(
+	SEXP handle,
+	SEXP y_data,
+	SEXP covariate_data,
+	SEXP num_covariates
+);
 
 /*!
 * \brief Calculate the value of the negative log-likelihood
@@ -802,12 +775,12 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_OptimLinRegrCoefCovPar_R(
 * \param[out] negll Negative log-likelihood
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_EvalNegLogLikelihood_R(
-	LGBM_SE handle,
-	LGBM_SE y_data,
-	LGBM_SE cov_pars,
-	LGBM_SE negll,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_EvalNegLogLikelihood_R(
+	SEXP handle,
+	SEXP y_data,
+	SEXP cov_pars,
+	SEXP negll
+);
 
 /*!
 * \brief Get covariance paramters
@@ -817,11 +790,11 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_EvalNegLogLikelihood_R(
 * \param[out] optim_cov_pars Optimal covariance parameters
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_GetCovPar_R(
-	LGBM_SE handle,
-	LGBM_SE calc_std_dev,
-	LGBM_SE optim_cov_pars,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_GetCovPar_R(
+	SEXP handle,
+	SEXP calc_std_dev,
+	SEXP optim_cov_pars
+);
 
 /*!
 * \brief Get initial values for covariance paramters
@@ -830,10 +803,10 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_GetCovPar_R(
 * \param[out] init_cov_pars Initial covariance parameters
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_GetInitCovPar_R(
-	LGBM_SE handle,
-	LGBM_SE init_cov_pars,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_GetInitCovPar_R(
+	SEXP handle,
+	SEXP init_cov_pars
+);
 
 /*!
 * \brief Get / export regression coefficients
@@ -843,11 +816,11 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_GetInitCovPar_R(
 * \param[out] optim_coef Optimal regression coefficients
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_GetCoef_R(
-	LGBM_SE handle,
-	LGBM_SE calc_std_dev,
-	LGBM_SE optim_coef,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_GetCoef_R(
+	SEXP handle,
+	SEXP calc_std_dev,
+	SEXP optim_coef
+);
 
 /*!
 * \brief Get / export the number of iterations until convergence
@@ -856,10 +829,10 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_GetCoef_R(
 * \param[out] num_it Number of iterations for convergence
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_GetNumIt_R(
-	LGBM_SE handle,
-	LGBM_SE num_it,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_GetNumIt_R(
+	SEXP handle,
+	SEXP num_it
+);
 
 /*!
 * \brief Set the data used for making predictions (useful if the same data is used repeatedly, e.g., in validation of GPBoost)
@@ -873,16 +846,16 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_GetNumIt_R(
 * \param covariate_data_pred Covariate data (=independent variables, features) for prediction
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_SetPredictionData_R(
-	LGBM_SE handle,
-	LGBM_SE num_data_pred,
-	LGBM_SE cluster_ids_data_pred,
-	LGBM_SE re_group_data_pred,
-	LGBM_SE re_group_rand_coef_data_pred,
-	LGBM_SE gp_coords_data_pred,
-	LGBM_SE gp_rand_coef_data_pred,
-	LGBM_SE covariate_data_pred,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_SetPredictionData_R(
+	SEXP handle,
+	SEXP num_data_pred,
+	SEXP cluster_ids_data_pred,
+	SEXP re_group_data_pred,
+	SEXP re_group_rand_coef_data_pred,
+	SEXP gp_coords_data_pred,
+	SEXP gp_rand_coef_data_pred,
+	SEXP covariate_data_pred
+);
 
 /*!
 * \brief Make predictions: calculate conditional mean and variances or covariance matrix
@@ -911,73 +884,61 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_SetPredictionData_R(
 * \param[out] out_predict Predictive/conditional mean at prediciton points followed by the predictive covariance matrix in column-major format (if predict_cov_mat==true) or the predictive variances (if predict_var==true)
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_PredictREModel_R(
-	LGBM_SE handle,
-	LGBM_SE y_data,
-	LGBM_SE num_data_pred,
-	LGBM_SE predict_cov_mat,
-	LGBM_SE predict_var,
-	LGBM_SE predict_response,
-	LGBM_SE cluster_ids_data_pred,
-	LGBM_SE re_group_data_pred,
-	LGBM_SE re_group_rand_coef_data_pred,
-	LGBM_SE gp_coords_pred,
-	LGBM_SE gp_rand_coef_data_pred,
-	LGBM_SE cov_pars,
-	LGBM_SE covariate_data_pred,
-	LGBM_SE use_saved_data,
-	LGBM_SE vecchia_pred_type,
-	LGBM_SE num_neighbors_pred,
-	LGBM_SE fixed_effects,
-	LGBM_SE fixed_effects_pred,
-	LGBM_SE out_predict,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_PredictREModel_R(
+	SEXP handle,
+	SEXP y_data,
+	SEXP num_data_pred,
+	SEXP predict_cov_mat,
+	SEXP predict_var,
+	SEXP predict_response,
+	SEXP cluster_ids_data_pred,
+	SEXP re_group_data_pred,
+	SEXP re_group_rand_coef_data_pred,
+	SEXP gp_coords_pred,
+	SEXP gp_rand_coef_data_pred,
+	SEXP cov_pars,
+	SEXP covariate_data_pred,
+	SEXP use_saved_data,
+	SEXP vecchia_pred_type,
+	SEXP num_neighbors_pred,
+	SEXP fixed_effects,
+	SEXP fixed_effects_pred,
+	SEXP out_predict
+);
 
 /*!
 * \brief Get name of likelihood
-* \param ll_name Likelihood name
-* \return 0 when succeed, -1 when failure happens
+* \return R character vector (length=1) with likelihood name
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_GetLikelihoodName_R(
-	LGBM_SE handle,
-	LGBM_SE buf_len,
-	LGBM_SE actual_len,
-	LGBM_SE ll_name,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_GetLikelihoodName_R(
+	SEXP handle
+);
 
 /*!
 * \brief Get name of covariance parameter optimizer
-* \param name Optimizer name
-* \return 0 when succeed, -1 when failure happens
+* \return R character vector (length=1) with optimizer name
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_GetOptimizerCovPars_R(
-	LGBM_SE handle,
-	LGBM_SE buf_len,
-	LGBM_SE actual_len,
-	LGBM_SE name,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_GetOptimizerCovPars_R(
+	SEXP handle
+);
 
 /*!
 * \brief Get name of linear regression coefficients optimizer
-* \param name Optimizer name
-* \return 0 when succeed, -1 when failure happens
+* \return R character vector (length=1) with optimizer name
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_GetOptimizerCoef_R(
-	LGBM_SE handle,
-	LGBM_SE buf_len,
-	LGBM_SE actual_len,
-	LGBM_SE name,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_GetOptimizerCoef_R(
+	SEXP handle
+);
 
 /*!
 * \brief Set the type of likelihood
 * \param likelihood Likelihood name
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_SetLikelihood_R(
-	LGBM_SE handle,
-	LGBM_SE likelihood,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_SetLikelihood_R(
+	SEXP handle,
+	SEXP likelihood
+);
 
 /*!
 * \brief Return (last used) response variable data
@@ -985,10 +946,10 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_SetLikelihood_R(
 * \param[out] response_data Response variable data (memory needs to be preallocated)
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_GetResponseData_R(
-	LGBM_SE handle,
-	LGBM_SE response_data,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_GetResponseData_R(
+	SEXP handle,
+	SEXP response_data
+);
 
 /*!
 * \brief Return covariate data
@@ -996,9 +957,9 @@ LIGHTGBM_C_EXPORT LGBM_SE GPB_GetResponseData_R(
 * \param[out] covariate_data covariate data
 * \return 0 when succeed, -1 when failure happens
 */
-LIGHTGBM_C_EXPORT LGBM_SE GPB_GetCovariateData_R(
-	LGBM_SE handle,
-	LGBM_SE covariate_data,
-	LGBM_SE call_state);
+LIGHTGBM_C_EXPORT SEXP GPB_GetCovariateData_R(
+	SEXP handle,
+	SEXP covariate_data
+);
 
 #endif  // GPBOOST_R_H_
